@@ -1,9 +1,11 @@
+from ast import Call
 import torch
 from collections import OrderedDict
 
 from .basic_manifold_torch import basic_manifold_torch
 from torch.nn.parameter import Parameter
-
+from torch import Tensor, DeviceObjType
+from typing import List, Tuple, Optional, Callable
 
 class stiefel_torch(basic_manifold_torch):
     def __init__(self, var_shape, device = torch.device('cpu'), dtype = torch.float64) -> None:
@@ -28,50 +30,50 @@ class stiefel_torch(basic_manifold_torch):
 
 
 
-    def Phi(self, M):
+    def Phi(self, M: Tensor) -> Tensor:
         return (M + M.transpose(-2,-1) )/2
 
 
 
-    def A(self, X):
+    def A(self, X: Tensor) -> Tensor:
         XX = torch.matmul(X.transpose(-2,-1), X)
         return 1.5 * X - torch.matmul(X , (XX /2))
 
 
-    def JA(self, X, G):
+    def JA(self, X: Tensor, G: Tensor) -> Tensor:
         return torch.matmul(G , ( self.Ip - 0.5 * self.C(X) ))  - torch.matmul(X , self.Phi(torch.matmul(X.transpose(-2,-1), G)) )
 
-    def JA_transpose(self,X,G):
+    def JA_transpose(self,X: Tensor, G: Tensor) -> Tensor:
         # JA is self-adjoint
         return self.JA(X,G)
 
-    def hessA(self, X, gradf, D):
+    def hessA(self, X: Tensor, gradf: Tensor, D: Tensor) -> Tensor:
         return - torch.matmul(D , self.Phi( torch.matmul(X.transpose(-2,-1), gradf)  )) - torch.matmul(X , self.Phi( torch.matmul(D.transpose(-2,-1) , gradf)  )) - torch.matmul(gradf , self.Phi( torch.matmul(D.transpose(-2,-1) , X) ))
 
 
-    def JC(self, X, Lambda):
+    def JC(self, X: Tensor, Lambda: Tensor) -> Tensor: 
         return torch.matmul(2*X , self.Phi(Lambda))
 
     
-    def C(self, X):
+    def C(self, X: Tensor) -> Tensor:
         # return X.T @ X - self.Ip.to(device=X.device, dtype = X.dtype)
         return torch.matmul(X.transpose(-2,-1), X) - self.Ip
 
-    def C_quad_penalty(self, X):
+    def C_quad_penalty(self, X: Tensor):
         return torch.sum(self.C(X) ** 2)
 
 
-    def hess_feas(self, X, D):
+    def hess_feas(self, X: Tensor, D: Tensor):
         return torch.matmul(4*X , self.Phi( torch.matmul(X.transpose(-2,-1), D) )  ) + 2* torch.matmul(D , self.C(X))
 
     
 
 
 
-    def Feas_eval(self, X):
+    def Feas_eval(self, X: Tensor):
         return torch.linalg.norm( self.C(X).flatten() , 2)
 
-    def Init_point(self, Xinit = None):
+    def Init_point(self, Xinit: Tensor = None):
         if Xinit is None:
             Xinit = torch.randn(*self.var_shape).to(device = self.device, dtype = self.dtype)
         else:
@@ -85,13 +87,13 @@ class stiefel_torch(basic_manifold_torch):
         Xinit.requires_grad = True
         return Xinit
 
-    def Post_process(self,X):
+    def Post_process(self,X: Tensor):
         UX, SX, VX = torch.svd(X)
         return torch.matmul(UX, VX.transpose(-2,-1))
 
 
 
-    def generate_cdf_fun(self, obj_fun, beta):
+    def generate_cdf_fun(self, obj_fun : Callable, beta: float):
         def local_obj_fun(X):
             CX = self.C(X)
             AX = X - 0.5 * X@ CX
@@ -106,7 +108,7 @@ class stiefel_torch(basic_manifold_torch):
 
 
 
-    def generate_cdf_grad(self, obj_grad, beta):
+    def generate_cdf_grad(self, obj_grad: Callable, beta: float):
         def local_grad(X):
             CX = self.C(X)
             AX = X - 0.5 * torch.matmul(X,CX)
@@ -123,7 +125,7 @@ class stiefel_torch(basic_manifold_torch):
 
 
 
-    def generate_cdf_hess(self, obj_grad, obj_hess, beta):
+    def generate_cdf_hess(self, obj_grad: Callable, obj_hess: Callable, beta: float):
         def local_hess(X, D):
             CX = self.C(X)
             AX = X - 0.5 *  torch.matmul(X,CX)
@@ -150,7 +152,7 @@ class stiefel_torch(basic_manifold_torch):
 
 
 
-    def generate_cdf_hess_approx(self, obj_grad, obj_hess, beta):
+    def generate_cdf_hess_approx(self, obj_grad: Callable, obj_hess: Callable, beta: float):
         def local_hess(X, D):
             CX = self.C(X)
             AX = X - 0.5 *  torch.matmul(X,CX)
